      subroutine SASA(coord)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INCLUDE 'SIZES'

      COMMON /MOLKST/ NUMAT,NAT,NFIRST,NMIDLE,NLAST, NORBS, NELECS,
     1                NALPHA,NBETA,NCLOSE,NOPEN,NDUMY,FRACT
      common /SurfArea/ SASAs
      double precision :: COORD(3,NUMATM),SASAs(NUMATM)
      integer       :: numat, nat(numatm), nfirst(numatm),nmidle(numatm)
      integer       :: nlast(numatm), norbs,nelecs,nalpha,nbeta
      integer       :: nopen,ndumy,fract

!*************************************************************************
!    Molecular area calculations
!
!    I got this routine from Sally Rose at Wellcome.
!    She thinks it comes from the QCPE routine MOLAREA, but isn't sure
!    AR   is the contribution of each atom to the area of the molecule
!    AREA is the total accessible surface area of the molecule
!
!***************************************************************TC 21.7.88 ***
      double precision, allocatable :: atdis(:,:)
      double precision              :: stot
      double precision              :: area
      double precision, allocatable :: ra(:),arxx(:)
      integer,          allocatable :: nn(:)

      double precision              :: a, a11, a12, agl, aj, ann
      double precision              :: areaxx, arp, arxp, b,c,cc
      double precision              :: din, disc, factor
      integer                       :: i, in, j, ja,jb,k,l,n,nanh
      integer                       :: neigh, nnnnn
      double precision              :: p, q, rr, rrr, seg, t, temp
      double precision              :: tha, thet
      double precision              :: unit, xa, ya, za,dx,dy,dz,r
      double precision              :: rs(107)
      double precision              :: zlarj(30),zsmal(30)
      logical                       :: istop
      character(len=255)            :: fmt
      logical                       :: debug=.true.
      integer                       :: LocAllStatus
      double precision, allocatable :: phip(:,:)
      double precision, allocatable :: phin(:,:)
      double precision, parameter   :: pi= 3.14159265358979d0
      double precision, parameter   :: Rh2o=1.4d0

!*****************************************************************************
! Van der Waals Radii taken from:
!    A.Bondi, J.Phys.Chem. (1964),68,441)
! except the values for B,Ge,Sn and Sb, which are calculating using
! Paulings formula ( R(vdw)=R(covalent)+0.76)
!*****************************************************************************
      data rs/1.2d0,                                       1.4d0,              
     1  2.2d0,1.9d0,1.65d0,1.7d0,1.55d0,1.52d0,1.47d0,1.54d0,                 
     2  2.5d0,2.4d0,2.3d0,2.1d0,1.8d0, 1.8d0, 1.75d0,1.88d0,                  
     3  10*2.d0,2.0d0,2.0d0,2.0d0,1.98d0,1.85d0,1.9d0,1.85d0,2.02d0,          
     4  10*2.d0,2.0d0,2.0d0,2.0d0,2.16d0,2.12d0,2.06d0,1.98d0,2.16d0,         
     5  53*2.d0/

      if (debug) write(6,'(/" <> Molecular Surface Area")')

 
      istop=.false.
      in=30
      din=30.0d0
      unit=2.0d0*Pi/din
!     
! 1. Check if all vdW Radii are defined
!
      do i=1,numat
        SASAs(i) = 0.0d0
        if (rs(nat(i)).eq.0.0d0)then
           fmt = '(/," <> ",2a,/," <> ",a)'
           write (6,fmt) 'Van der Waals Radii are not ',                          
     .    'available for some elements; Molecular Area not calculated.'
           return
        endif
      enddo
!
! Allocate arrays
!
      LocAllStatus = 0
      if(allocated(phip))  deallocate(phip)
      if(allocated(phin))  deallocate(phin)
      if(allocated(atdis)) deallocate(atdis)
      if(allocated(ra))    deallocate(ra)
      if(allocated(arxx))  deallocate(arxx)
      if(allocated(nn))    deallocate(nn)
      Allocate(phip(numat,30),phin(numat,30),atdis(numat,numat),
     1         ra(numat),arxx(numat),nn(numat),
     2         stat=LocAllStatus)
      if(LocAllStatus.ne.0) then
         write(6,'(" <!> Allocation problem in SASA <!>")')
         stop
      endif
!
! Calculate atdis
!
      atdis(1:numat,1:numat) = 0.0d0
      do i=2,numat
         do j=1,i-1
           dx = coord(1,i) - coord(1,j)
           dy = coord(2,i) - coord(2,j)
           dz = coord(3,i) - coord(3,j)
! Unprotected because MOPAC has already approved the coordinates
           r = sqrt(dx*dx + dy*dy + dz*dz)
           atdis(i,j) = r
           atdis(j,i) = r
        enddo
      enddo

      CalculationLoop : do
!
! Add a solvent radius if SAS area is requested
!
      do i=1,numat
         ra(i)=rs(nat(i))+rh2o
      enddo

      nanh=numat
      n=nanh
      do i=1,n
        area=4.0d0*Pi*ra(i)*ra(i)
        areaxx=area
        neigh=n
        do j=1,neigh
          rr=atdis(i,j)
          do k=1,in
            zlarj(k)=ra(i)
            zsmal(k)=ra(i)
            phip(j,k)=ra(i)
            phin(j,k)=ra(i)
          enddo

          nn(j)=j
          nnnnn=nn(j)
!      
! Skip atom j if not a neighbor
!
         if (i==j) cycle         
         if ((ra(i)+ra(nnnnn)-rr) < 0.0d0) cycle

         temp = ra(i)*ra(i) + rr*rr - ra(nnnnn)*ra(nnnnn)
         temp = temp/(2.0d0*ra(i)*rr)
         if (temp.lt.-1.0d0) temp=-1.0d0
         if (temp.gt. 1.0d0) temp=1.0d0
         aj=acos(temp)

         xa=coord(1,nnnnn)-coord(1,i)
         ya=coord(2,nnnnn)-coord(2,i)
         za=coord(3,nnnnn)-coord(3,i)

         tha=0.0d0
         t=sqrt(xa*xa+ya*ya)
         if (t.eq.0.0d0)then
           temp=sign(1.0d0,xa)
         else
           temp=xa/t
           if (temp.lt.-1.0d0) temp=-1.0d0
           if (temp.gt. 1.0d0) temp= 1.0d0
         endif
   
         if (t .ne.0.0d0) tha=acos(temp)
        if (ya.le.0.0d0) tha=2.0d0*Pi-tha

         temp=za/rr
         if (temp.lt.-1.0d0) temp=-1.0d0
         if (temp.gt. 1.0d0) temp= 1.0d0
         ann=acos(temp)
         xa=xa*ra(i)*cos(aj)/rr
         ya=ya*ra(i)*cos(aj)/rr
         za=za*ra(i)*cos(aj)/rr
         do k=1,in
           agl=k*unit
           p=(xa*xa+ya*ya+za*za)
           q=(xa*cos(agl)+ya*sin(agl))
           a=q*q+za*za
           b=-2.0d0*za*p
           c=p*p-ra(i)*ra(i)*q*q

           if(abs(a).lt.0.00001d0) cycle

           disc=b*b-4.0d0*a*c
           if (disc.ge.0.0d0) then
             a11=(-b+sqrt(disc))/(2.0d0*a)
             a12=(-b-sqrt(disc))/(2.0d0*a)
             zlarj(k)=max(a11,a12)
             zsmal(k)=min(a11,a12)
           else
             if (abs(disc).le.0.001d0) then
               zlarj(k)=-b/(2.0d0*a)
               zsmal(k)=zlarj(k)
           endif
         endif
      enddo

      do k=1,in
        thet=k*unit
        if (coord(3,nnnnn)-coord(3,i).ge.0.0d0) then
          if (aj.le.ann) then
            if (thet-tha.le.0.0d0) then
              cc=cos(thet-tha+2.0d0*Pi)
            else
              cc=cos(thet-tha)
            endif

            if (cc.ge.0.0d0) then
              phip(j,k)=zlarj(k)
              phin(j,k)=zsmal(k)
            endif
          else
            if (Pi-aj.ge.ann) then
              if(thet-tha.lt.0.0d0)then
                cc=cos(thet-tha+2.0d0*Pi)
              else
                cc=cos(thet-tha)
              endif
        
              if(cc.gt.0)then
                phin(j,k)=zsmal(k)
              else
                phin(j,k)=zlarj(k)
              endif
              phip(j,k)=ra(i)
            else
              phip(j,k)=ra(i)
              phin(j,k)=-ra(i)
              if (zlarj(k).ge.ra(i)) then
                zlarj(k)=-ra(i)
                zsmal(k)=ra(i)
              endif
                if (thet-tha.le.0.0d0)then
                   cc=cos(thet-tha+2.0d0*Pi)
                else
                   cc=cos(thet-tha)
                endif
                if (cc.le.0.0d0) then
                  phip(j,k)=zsmal(k)
                  phin(j,k)=zlarj(k)
                endif
              endif
            endif
          else
            if (aj.le.(Pi-ann)) then
              if (thet-tha.le.0.0d0) then
                cc=cos(thet-tha+2.0d0*Pi)
              else
                cc=cos(thet-tha)
              endif
              if (cc.ge.0.0d0) then
                phip(j,k)=zlarj(k)
                phin(j,k)=zsmal(k)
              endif
            else
              if (aj.le.ann) then
                if (thet-tha.lt.0.0d0)then
                  cc=cos(thet-tha+2.0d0*Pi)
                else
                  cc=cos(thet-tha)
                endif
              if (cc.gt.0.0d0)then
                phip(j,k)=zlarj(k)
              else
                phip(j,k)=zsmal(k)
              endif
              phin(j,k)=-ra(i)
            else
              phip(j,k)=ra(i)
              phin(j,k)=-ra(i)
              if (zlarj(k).ge.ra(i)) then
                zlarj(k)=-ra(i)
                zsmal(k)=ra(i)
             endif
             if(thet-tha.gt.0.0d0)then
               cc=cos(thet-tha+2.0d0*Pi)
             else
               cc=cos(thet-tha)
             endif
             if (cc.le.0.0d0) then
               phip(j,k)=zsmal(k)
               phin(j,k)=zlarj(k)
             endif
           endif
         endif
       endif
      enddo

      area=0.0d0
      l=j
      enddo

      do jb=1,neigh
         if(jb.eq.i) cycle

           rr=atdis(i,jb)
           do ja=jb+1,neigh
             if (ja.eq.i) cycle
             rrr=atdis(i,ja)
             if(ra(i)+ra(ja)-rrr.lt.0.0d0) cycle
             if(ra(i)+ra(jb)-rr.lt.0.0d0) cycle

             do k=1,in
               if (phip(ja,k).ge.phin(ja,k)) then
                 if (phip(jb,k).ge.phin(jb,k)) then
                   if (phip(jb,k).gt.phip(ja,k)) then
                     if (phin(jb,k).gt.phip(ja,k)) then
                       continue
                     elseif(phin(jb,k).lt.phin(ja,k)) then
                       phip(ja,k)=phip(jb,k)
                       phin(ja,k)=phin(jb,k)
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     else
                       phip(ja,k)=phip(jb,k)
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     endif
                   elseif (phip(jb,k).lt.phin(ja,k)) then
                     continue
                     elseif (phin(jb,k).lt.phin(ja,k)) then
                       phin(ja,k)=phin(jb,k)
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     else
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     endif
                   elseif (phip(ja,k).gt.phin(jb,k)) then
                     if (phin(ja,k).lt.phin(jb,k)) then
                       if(phin(ja,k).gt.phip(jb,k)) then
                         phip(ja,k)=phip(jb,k)
                         phip(jb,k)=ra(i)
                         phin(jb,k)=ra(i)
                       else
                         phin(ja,k)=-ra(i)
                         phip(ja,k)=ra(i)
                         phip(jb,k)=ra(i)
                         phin(jb,k)=ra(i)
                       endif
                     else
                       phin(ja,k)=phin(jb,k)
                       phip(ja,k)=phip(jb,k)
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     endif
                   elseif (phip(ja,k).gt.phip(jb,k)) then
                     if (phin(ja,k).gt.phip(jb,k)) then
                       continue
                     else
                       phin(ja,k)=phin(jb,k)
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                     endif
                   else
                     phip(ja,k)=phip(jb,k)
                     phin(ja,k)=phin(jb,k)
                     phip(jb,k)=ra(i)
                     phin(jb,k)=ra(i)
                   endif
                elseif(phip(jb,k).ge.phin(jb,k)) then
                   if (phip(jb,k).gt.phin(ja,k)) then
                     if (phin(jb,k).lt.phin(ja,k)) then
                       if(phin(jb,k).gt.phip(ja,k)) then
                         phin(ja,k)=phin(jb,k)
                         phip(jb,k)=ra(i)
                         phin(jb,k)=ra(i)
                       else
                         phip(ja,k)=ra(i)
                         phin(ja,k)=-ra(i)
                         phip(jb,k)=ra(i)
                         phin(jb,k)=ra(i)
                       endif
                     else
                       phip(jb,k)=ra(i)
                       phin(jb,k)=ra(i)
                    endif
                  elseif(phip(jb,k).gt.phip(ja,k)) then
                    if (phin(jb,k).gt.phip(ja,k)) then
                      continue
                    else
                      phip(ja,k)=phip(jb,k)
                      phip(jb,k)=ra(i)
                      phin(jb,k)=ra(i)
                    endif
                  else
                    phip(jb,k)=ra(i)
                    phin(jb,k)=ra(i)
                  endif
                elseif(phin(ja,k).gt.phin(jb,k)) then
                  if (phip(ja,k).gt.phin(jb,k)) then
                    phip(ja,k)=ra(i)
                    phin(ja,k)=-ra(i)
                    phip(jb,k)=ra(i)
                    phin(jb,k)=ra(i)
                  elseif (phip(ja,k).lt.phip(jb,k)) then
                    phin(ja,k)=phin(jb,k)
                    phip(ja,k)=phip(jb,k)
                    phip(jb,k)=ra(i)
                    phin(jb,k)=ra(i)
                  else
                    phin(ja,k)=phin(jb,k)
                    phip(jb,k)=ra(i)
                    phin(jb,k)=ra(i)
                  endif
                elseif(phin(ja,k).lt.phip(jb,k)) then
                  phip(ja,k)=ra(i)
                  phin(ja,k)=-ra(i)
                  phip(jb,k)=ra(i)
                  phin(jb,k)=ra(i)
                elseif(phip(ja,k).lt.phip(jb,k)) then
                  phip(ja,k)=phip(jb,k)
                  phip(jb,k)=ra(i)
                  phin(jb,k)=ra(i)
                else
                  phip(jb,k)=ra(i)
                  phin(jb,k)=ra(i)
                endif
              enddo
            enddo
          enddo

          area=0.0d0
         do k=1,in
           do j=1,neigh
             if(j.eq.i) cycle
             rr=atdis(i,j)
             if(ra(i)+ra(j)-rr.lt.0.0d0) cycle
             seg=phip(j,k)-phin(j,k)
             if (seg .lt. 0.0d0) then
               seg=seg+2.0d0*ra(i)
             endif
             area=area+seg
           enddo
         enddo

         area=2.0d0*Pi*ra(i)*area/din
         area=4.0d0*Pi*ra(i)*ra(i)-area
         SASAs(i)=area
         arxx(i)=areaxx
       enddo

       area=0.0d0
       do i=1,n
         area=area+SASAs(i) 
       enddo

       if (debug) then
         if(rh2o.eq.0)then
           fmt = '(/4x,"van der Waals Surface :")'
           write(6,fmt)
           fmt = '( 4x,a," : ",f9.3," square Angstroms")'
           write(6,fmt) 'Accessible Molecular Surface Area',area
         else
           fmt = '(/4x,"Solvent Accessible Surface :")'
           write(6,fmt)
           fmt = '( 4x,"Solvent radius of ",f6.3," Angstroms")'
           write(6,fmt) rh2o
           fmt = '( 4x,a," : ",f9.3," square Angstroms")'
           write(6,fmt) 'Area at center of the solvent sphere',area
         endif
       else
         stot=area
       endif

       if (rh2o.eq.0)then
         if (debug) then
           fmt='(/6x,"Atom",6x,"Maximum Area",5x,"Accessible Area")'
           write (6,fmt)
           fmt='(4x,i5,4x,f12.3,5x,f12.3)'
           write(6,fmt) (k,arxx(k),SASAs(k),k=1,n)
         endif
       else
         if(debug) then
           fmt='(/6x,"Atom",11x,"Maximum Area",18x,"Accessible Area")'
           write (6,fmt)
           fmt='(9x,2(8x,"solvent"3x,"van der Waals"))'
           write (6,fmt)
           fmt='(6x,2(12x,"center",6x,"surface"))'
           write (6,fmt)
           fmt='(4x,i5,2(6x,f9.3,3x,f9.3,:,4x))'
           do k=1,n
             factor = rs(nat(k))*rs(nat(k))/(ra(k)*ra(k))
             arp = SASAs(k)*factor
            arxp= arxx(k)*factor
             write(6,fmt)k,arxx(k),arxp,SASAs(k),arp
           enddo
         endif
       endif
         
       exit CalculationLoop
       if (istop) exit CalculationLoop

       istop=.true.
      enddo CalculationLoop
!
! Dellocate arrays (M.Hennemann)
!
      LocAllStatus = 0
      if(allocated(phin))  Deallocate(phin, stat=LocAllStatus)
      if(allocated(phip))  Deallocate(phip, stat=LocAllStatus)
      if(allocated(atdis)) Deallocate(atdis,stat=LocAllStatus)
      if(allocated(arxx))  Deallocate(arxx, stat=LocAllStatus)
      if(allocated(ra))    Deallocate(ra,   stat=LocAllStatus)
      if(allocated(nn))    Deallocate(nn,   stat=LocAllStatus)

      return
      end 
